/* -*-c-*-
 *
 * $Id: tripe.h,v 1.15 2003/05/16 12:09:03 mdw Exp $
 *
 * Main header file for TrIPE
 *
 * (c) 2001 Straylight/Edgeware
 */

/*----- Licensing notice --------------------------------------------------* 
 *
 * This file is part of Trivial IP Encryption (TrIPE).
 *
 * TrIPE is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * TrIPE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with TrIPE; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/*----- Revision history --------------------------------------------------* 
 *
 * $Log: tripe.h,v $
 * Revision 1.15  2003/05/16 12:09:03  mdw
 * Allow binding to a chosen address.
 *
 * Revision 1.14  2003/04/06 10:36:33  mdw
 * Rearrange so as not to include Linux headers unless we need to.
 *
 * Revision 1.13  2003/04/06 10:26:35  mdw
 * Report peer name on decrypt errors.
 *
 * Revision 1.12  2003/04/06 10:25:17  mdw
 * Support Linux TUN/TAP device.  Fix some bugs.
 *
 * Revision 1.11  2002/01/13 14:57:42  mdw
 * Fix crap typo.
 *
 * Revision 1.10  2002/01/13 14:54:58  mdw
 * Provide MGF macros.
 *
 * Revision 1.9  2001/06/22 19:40:36  mdw
 * Support expiry of other peers' public keys.
 *
 * Revision 1.8  2001/06/19 22:10:57  mdw
 * Some more constants for the algorithms.  Document the packet format
 * change for non-malleability.  Moved @buf@ definitions to separate header
 * file.
 *
 * Revision 1.7  2001/03/03 12:07:08  mdw
 * Rename word get and put functions now that there's 16-bit support.
 *
 * Revision 1.6  2001/02/19 19:11:09  mdw
 * Output buffering on admin connections.
 *
 * Revision 1.5  2001/02/16 21:41:43  mdw
 * Major changes.  See source files for details.
 *
 * Revision 1.4  2001/02/05 19:56:37  mdw
 * Sequence number protection, and BSD tunnels.
 *
 * Revision 1.3  2001/02/04 01:17:55  mdw
 * Create a configuration header file to tidy up command lines.
 *
 * Revision 1.2  2001/02/03 22:40:29  mdw
 * Put timer information into the entropy pool when packets are received
 * and on similar events.  Reseed the generator on the interval timer.
 *
 * Revision 1.1  2001/02/03 20:26:37  mdw
 * Initial checkin.
 *
 */

#ifndef TRIPE_H
#define TRIPE_H

#ifdef __cplusplus
  extern "C" {
#endif

/*----- Header files ------------------------------------------------------*/

#include "config.h"

#include <assert.h>
#include <ctype.h>
#include <errno.h>
#include <signal.h>
#include <stdarg.h>
#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include <sys/types.h>
#include <sys/time.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/stat.h>

#include <sys/socket.h>
#include <sys/un.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>

#include <pwd.h>
#include <grp.h>

#include <mLib/alloc.h>
#include <mLib/arena.h>
#include <mLib/bres.h>
#include <mLib/dstr.h>
#include <mLib/env.h>
#include <mLib/fdflags.h>
#include <mLib/fwatch.h>
#include <mLib/mdwopt.h>
#include <mLib/quis.h>
#include <mLib/report.h>
#include <mLib/sel.h>
#include <mLib/selbuf.h>
#include <mLib/sig.h>
#include <mLib/str.h>
#include <mLib/sub.h>
#include <mLib/trace.h>

#include <catacomb/gcipher.h>
#include <catacomb/gmac.h>
#include <catacomb/grand.h>
#include <catacomb/key.h>
#include <catacomb/paranoia.h>

#include <catacomb/noise.h>
#include <catacomb/rand.h>

#include <catacomb/mp.h>
#include <catacomb/mpmont.h>
#include <catacomb/mprand.h>
#include <catacomb/dh.h>

#include "buf.h"
#include "util.h"

#undef sun

/*----- Magic numbers -----------------------------------------------------*/

/* --- Tunnel types --- */

#define TUN_NOTDEF 0
#define TUN_UNET 1
#define TUN_BSD 2
#define TUN_LINUX 3

/* --- Trace flags --- */

#define T_TUNNEL 1u
#define T_PEER 2u
#define T_PACKET 4u
#define T_ADMIN 8u
#define T_CRYPTO 16u
#define T_KEYSET 32u
#define T_KEYEXCH 64u
#define T_KEYMGMT 128u

#define T_ALL 255u

/* --- Units --- */

#define SEC(n) (n##u)
#define MIN(n) (n##u * 60u)
#define MEG(n) (n##ul * 1024ul * 1024ul)

/* --- Other things --- */

#define PKBUFSZ 65536

/*----- TrIPE protocol ----------------------------------------------------*/

/* --- TrIPE message format --- *
 *
 * A packet begins with a single-byte message type.  The top four bits are a
 * category code used to send the message to the right general place in the
 * code; the bottom bits identify the actual message type.
 */

#define MSG_CATMASK 0xf0
#define MSG_TYPEMASK 0x0f

/* --- Encrypted message packets --- *
 *
 * Messages of category @MSG_PACKET@ contain encrypted network packets.  The
 * message content is a symmetric-encrypted block (see below).  Reception of
 * a packet encrypted under a new key implicitly permits that key to be used
 * to send further packets.
 *
 * The only packet type accepted is zero.
 *
 * Packets may be encrypted under any live keyset, but should use the most
 * recent one.
 */

#define MSG_PACKET 0x00

/* --- Key exchange packets --- */

#define MSG_KEYEXCH 0x10

#define KX_PRECHAL 0u
#define KX_COOKIE 1u
#define KX_CHAL 2u
#define KX_REPLY 3u
#define KX_SWITCH 4u
#define KX_SWITCHOK 5u
#define KX_NMSG 6u

/* --- Symmetric encryption and keysets --- *
 *
 * Packets consist of an 80-bit MAC, a 32-bit sequence number, and the
 * encrypted payload.
 *
 * The plaintext is encrypted using Blowfish in CBC mode with ciphertext
 * stealing (as described in [Schneier].  The initialization vector is
 * selected randomly, and prepended to the actual ciphertext.
 *
 * The MAC is computed using the HMAC construction with RIPEMD160 over the
 * sequence number and the ciphertext (with IV); the first 80 bits of the
 * output are used.  (This is the minimum allowed by the draft FIPS for HMAC,
 * and the recommended truncation.)
 *
 * A keyset consists of
 *
 *   * an integrity (MAC) key;
 *   * a confidentiality (encryption) key; and
 *   * a sequence numbering space
 *
 * in each direction.  The packets sent by a host encrypted under a
 * particular keyset are assigned consecutive sequence numbers starting from
 * zero.  The receiving host must ensure that it only accepts each packet at
 * most once.  It should maintain a window of sequence numbers: packets with
 * numbers beyond the end of the window are accepted and cause the window to
 * be advanced; packets with numbers before the start of the window are
 * rejected; packets with numbers which appear within the window are accepted
 * only if the number has not been seen before.
 *
 * When a host sends a @KX_SWITCH@ or @KX_SWITCHOK@ message, it installs the
 * newly-negotiated keyset in a `listen-only' state: it may not send a packet
 * encrypted under the keyset until either it has received a @KX_SWITCH@ or
 * @KX_SWITCHOK@ message, or a @MSG_PACKET@ encrypted under the keyset, from
 * its peer.
 */

/*----- Cipher selections -------------------------------------------------*/

#include <catacomb/blowfish.h>
#include <catacomb/blowfish-cbc.h>
#include <catacomb/blowfish-counter.h>
#include <catacomb/rmd160.h>
#include <catacomb/rmd160-hmac.h>

#define CIPHER (&blowfish_cbc)
#define MAC (&rmd160_hmac)

#define HASH_CTX rmd160_ctx
#define HASH_INIT rmd160_init
#define HASH rmd160_hash
#define HASH_STRING(c, s) HASH((c), s, sizeof(s))
#define HASH_DONE rmd160_done
#define HASHSZ RMD160_HASHSZ

#define MGF_CTX blowfish_counterctx
#define MGF_INIT blowfish_counterinit
#define MGF_CRYPT blowfish_counterencrypt

#define SEQSZ 4
#define IVSZ BLOWFISH_BLKSZ
#define MACSZ 10

/*----- Data structures ---------------------------------------------------*/

/* --- Socket addresses --- *
 *
 * A magic union of supported socket addresses.
 */

typedef union addr {
  struct sockaddr sa;
  struct sockaddr_in sin;
} addr;

/* --- A symmetric keyset --- *
 *
 * A keyset contains a set of symmetric keys for encrypting and decrypting
 * packets.  Keysets are stored in a list, sorted in reverse order of
 * creation, so that the most recent keyset (the one most likely to be used)
 * is first.
 *
 * Each keyset has a time limit and a data limit.  The keyset is destroyed
 * when either it has existed for too long, or it has been used to encrypt
 * too much data.  New key exchanges are triggered when keys are close to
 * expiry.
 */

typedef struct keyset {
  struct keyset *next;			/* Next active keyset in the list */
  unsigned ref;				/* Reference count for keyset */
  struct peer *p;			/* Pointer to peer structure */
  time_t t_exp;				/* Expiry time for this keyset */
  unsigned long sz_exp;			/* Data limit for the keyset */
  T( unsigned seq; )			/* Sequence number for tracing */
  unsigned f;				/* Various useful flags */
  gcipher *cin, *cout; 			/* Keyset ciphers for encryption */
  gmac *min, *mout;	       		/* Keyset MACs for integrity */
  uint32 oseq;				/* Outbound sequence number */
  uint32 iseq, iwin;			/* Inbound sequence number */
} keyset;

#define KS_SEQWINSZ 32			/* Bits in sequence number window */

#define KSF_LISTEN 1u			/* Don't encrypt packets yet */
#define KSF_LINK 2u			/* Key is in a linked list */

/* --- Key exchange --- *
 *
 * TrIPE uses the Wrestlers Protocol for its key exchange.  The Wrestlers
 * Protocol has a number of desirable features (e.g., perfect forward
 * secrecy, and zero-knowledge authentication) which make it attractive for
 * use in TrIPE.  The Wrestlers Protocol was designed by Mark Wooding and
 * Clive Jones.
 */

#define KX_NCHAL 16u
#define KX_THRESH 4u

typedef struct kxchal {
  struct keyexch *kx;			/* Pointer back to key exchange */
  mp *c;				/* Responder's challenge */
  mp *r;				/* My reply to the challenge */
  keyset *ks;				/* Pointer to temporary keyset */
  unsigned f;				/* Various useful flags */
  sel_timer t;				/* Response timer for challenge */
  octet hc[HASHSZ];			/* Hash of his challenge */
  mp *ck;				/* The check value */
  octet hswrq_in[HASHSZ];		/* Inbound switch request message */
  octet hswok_in[HASHSZ];		/* Inbound switch confirmation */
  octet hswrq_out[HASHSZ];		/* Outbound switch request message */
  octet hswok_out[HASHSZ];		/* Outbound switch confirmation */
} kxchal;

typedef struct keyexch {
  struct peer *p;			/* Pointer back to the peer */
  keyset **ks;				/* Peer's list of keysets */
  unsigned f;				/* Various useful flags */
  unsigned s;				/* Current state in exchange */
  sel_timer t;				/* Timer for next exchange */
  dh_pub kpub;				/* Peer's public key */
  time_t texp_kpub;			/* Expiry time for public key */
  mp *alpha;				/* My temporary secret */
  mp *c;				/* My challenge */
  mp *rx;				/* The expected response */
  unsigned nr;				/* Number of extant responses */
  time_t t_valid;			/* When this exchange goes bad */
  octet hc[HASHSZ];			/* Hash of my challenge */
  kxchal *r[KX_NCHAL];			/* Array of challenges */
} keyexch;

#define KXF_TIMER 1u			/* Waiting for a timer to go off */
#define KXF_DEAD 2u			/* The key-exchanger isn't up */
#define KXF_PUBKEY 4u			/* Key exchanger has a public key */

enum {
  KXS_DEAD,				/* Uninitialized state (magical) */
  KXS_CHAL,				/* Main answer-challenges state */
  KXS_COMMIT,				/* Committed: send switch request */
  KXS_SWITCH				/* Switched: send confirmation */
};

/* --- Tunnel structure --- *
 *
 * Used to maintain system-specific information about the tunnel interface.
 */

#if TUN_TYPE == TUN_LINUX
#  include <linux/if.h>
#  include <linux/if_tun.h>
#endif

typedef struct tunnel {
#if TUN_TYPE == TUN_UNET 
  sel_file f;				/* Selector for Usernet device */
  struct peer *p;			/* Pointer to my peer */
#elif TUN_TYPE == TUN_LINUX
  sel_file f;				/* Selector for TUN/TAP device */
  struct peer *p;			/* Pointer to my peer */
  char ifn[IFNAMSIZ];			/* Interface name buffer */
#elif TUN_TYPE == TUN_BSD
  sel_file f;				/* Selector for tunnel device */
  struct peer *p;			/* Pointer to my peer */
  unsigned n;				/* Number of my tunnel device */
#else
#  error "No support for this tunnel type"
#endif
} tunnel;

/* --- Peer statistics --- *
 *
 * Contains various interesting and not-so-interesting statistics about a
 * peer.  This is updated by various parts of the code.  The format of the
 * structure isn't considered private, and @p_stats@ returns a pointer to the
 * statistics block for a given peer.
 */

typedef struct stats {
  unsigned long sz_in, sz_out;		/* Size of all data in and out */
  unsigned long sz_kxin, sz_kxout;	/* Size of key exchange messages */
  unsigned long sz_ipin, sz_ipout;	/* Size of encapsulated IP packets */
  time_t t_start, t_last;		/* Time peer created, last recv */
  unsigned long n_reject;		/* Number of rejected packets */
  unsigned long n_in, n_out;		/* Number of packets in and out */
  unsigned long n_kxin, n_kxout;	/* Number of key exchange packets */
  unsigned long n_ipin, n_ipout;	/* Number of encrypted packets */
} stats;

/* --- Peer structure --- *
 *
 * The main structure which glues everything else together.
 */

typedef struct peer {
  struct peer *next, *prev;		/* Links to next and previous */
  char *name;				/* Name of this peer */
  tunnel t;				/* Tunnel for local packets */
  keyset *ks;				/* List head for keysets */
  buf b;				/* Buffer for sending packets */
  addr peer;				/* Peer socket address */
  size_t sasz;				/* Socket address size */
  stats st;				/* Statistics */
  keyexch kx;				/* Key exchange protocol block */
} peer;

/* --- Admin structure --- */

#define OBUFSZ 16384u

typedef struct obuf {
  struct obuf *next;			/* Next buffer in list */
  char *p_in, *p_out;			/* Pointers into the buffer */
  char buf[OBUFSZ];			/* The actual buffer */
} obuf;

typedef struct admin {
  struct admin *next, *prev;		/* Links to next and previous */
  unsigned f;				/* Various useful flags */
#ifndef NTRACE
  unsigned seq;				/* Sequence number for tracing */
#endif
  char *pname;				/* Peer name to create */
  char *paddr;				/* Address string to resolve */
  obuf *o_head, *o_tail;		/* Output buffer list */
  selbuf b;				/* Line buffer for commands */
  sel_file w;				/* Selector for write buffering */
  bres_client r;			/* Background resolver task */
  sel_timer t;				/* Timer for resolver */
  addr peer;				/* Address to set */
  size_t sasz;				/* Size of the address */
} admin;

#define AF_DEAD 1u			/* Destroy this admin block */
#define AF_LOCK 2u			/* Don't destroy it yet */

/*----- Global variables --------------------------------------------------*/

extern sel_state sel;			/* Global I/O event state */
extern dh_priv kpriv;			/* Our private key */
extern mpmont mg;			/* Montgomery context for DH group */
extern octet buf_i[PKBUFSZ], buf_o[PKBUFSZ], buf_t[PKBUFSZ];

#ifndef NTRACE
extern const trace_opt tr_opts[];	/* Trace options array */
extern unsigned tr_flags;		/* Trace options flags */
#endif

/*----- Other macros ------------------------------------------------------*/

#define TIMER noise_timer(RAND_GLOBAL)

/*----- Key management ----------------------------------------------------*/

/* --- @km_interval@ --- *
 *
 * Arguments:	---
 *
 * Returns:	Zero if OK, nonzero to force reloading of keys.
 *
 * Use:		Called on the interval timer to perform various useful jobs.
 */

extern int km_interval(void);

/* --- @km_init@ --- *
 *
 * Arguments:	@const char *kr_priv@ = private keyring file
 *		@const char *kr_pub@ = public keyring file
 *		@const char *tag@ = tag to load
 *
 * Returns:	---
 *
 * Use:		Initializes, and loads the private key.
 */

extern void km_init(const char */*kr_priv*/, const char */*kr_pub*/,
		    const char */*tag*/);

/* --- @km_getpubkey@ --- *
 *
 * Arguments:	@const char *tag@ = public key tag to load
 *		@dh_pub *kpub@ = where to put the public key
 *		@time_t *t_exp@ = where to put the expiry time
 *
 * Returns:	Zero if OK, nonzero if it failed.
 *
 * Use:		Fetches a public key from the keyring.
 */

extern int km_getpubkey(const char */*tag*/, dh_pub */*kpub*/,
			time_t */*t_exp*/);

/*----- Key exchange ------------------------------------------------------*/

/* --- @kx_start@ --- *
 *
 * Arguments:	@keyexch *kx@ = pointer to key exchange context
 *
 * Returns:	---
 *
 * Use:		Stimulates a key exchange.  If a key exchage is in progress,
 *		a new challenge is sent (unless the quiet timer forbids
 *		this); if no exchange is in progress, one is commenced.
 */

extern void kx_start(keyexch */*kx*/);

/* --- @kx_message@ --- *
 *
 * Arguments:	@keyexch *kx@ = pointer to key exchange context
 *		@unsigned msg@ = the message code
 *		@buf *b@ = pointer to buffer containing the packet
 *
 * Returns:	---
 *
 * Use:		Reads a packet containing key exchange messages and handles
 *		it.
 */

extern void kx_message(keyexch */*kx*/, unsigned /*msg*/, buf */*b*/);

/* --- @kx_free@ --- *
 *
 * Arguments:	@keyexch *kx@ = pointer to key exchange context
 *
 * Returns:	---
 *
 * Use:		Frees everything in a key exchange context.
 */

extern void kx_free(keyexch */*kx*/);

/* --- @kx_newkeys@ --- *
 *
 * Arguments:	@keyexch *kx@ = pointer to key exchange context
 *
 * Returns:	---
 *
 * Use:		Informs the key exchange module that its keys may have
 *		changed.  If fetching the new keys fails, the peer will be
 *		destroyed, we log messages and struggle along with the old
 *		keys.
 */

extern void kx_newkeys(keyexch */*kx*/);

/* --- @kx_init@ --- *
 *
 * Arguments:	@keyexch *kx@ = pointer to key exchange context
 *		@peer *p@ = pointer to peer context
 *		@keyset **ks@ = pointer to keyset list
 *
 * Returns:	Zero if OK, nonzero if it failed.
 *
 * Use:		Initializes a key exchange module.  The module currently
 *		contains no keys, and will attempt to initiate a key
 *		exchange.
 */

extern int kx_init(keyexch */*kx*/, peer */*p*/, keyset **/*ks*/);

/*----- Keysets and symmetric cryptography --------------------------------*/

/* --- @ks_drop@ --- *
 *
 * Arguments:	@keyset *ks@ = pointer to a keyset
 *
 * Returns:	---
 *
 * Use:		Decrements a keyset's reference counter.  If the counter hits
 *		zero, the keyset is freed.
 */

extern void ks_drop(keyset */*ks*/);

/* --- @ks_gen@ --- *
 *
 * Arguments:	@const void *k@ = pointer to key material
 *		@size_t x, y, z@ = offsets into key material (see below)
 *		@peer *p@ = pointer to peer information
 *
 * Returns:	A pointer to the new keyset.
 *
 * Use:		Derives a new keyset from the given key material.  The
 *		offsets @x@, @y@ and @z@ separate the key material into three
 *		parts.  Between the @k@ and @k + x@ is `my' contribution to
 *		the key material; between @k + x@ and @k + y@ is `your'
 *		contribution; and between @k + y@ and @k + z@ is a shared
 *		value we made together.  These are used to construct two
 *		pairs of symmetric keys.  Each pair consists of an encryption
 *		key and a message authentication key.  One pair is used for
 *		outgoing messages, the other for incoming messages.
 *
 *		The new key is marked so that it won't be selected for output
 *		by @ksl_encrypt@.  You can still encrypt data with it by
 *		calling @ks_encrypt@ directly.
 */

extern keyset *ks_gen(const void */*k*/,
		      size_t /*x*/, size_t /*y*/, size_t /*z*/,
		      peer */*p*/);

/* --- @ks_tregen@ --- *
 *
 * Arguments:	@keyset *ks@ = pointer to a keyset
 *
 * Returns:	The time at which moves ought to be made to replace this key.
 */

extern time_t ks_tregen(keyset */*ks*/);

/* --- @ks_activate@ --- *
 *
 * Arguments:	@keyset *ks@ = pointer to a keyset
 *
 * Returns:	---
 *
 * Use:		Activates a keyset, so that it can be used for encrypting
 *		outgoing messages.
 */

extern void ks_activate(keyset */*ks*/);

/* --- @ks_encrypt@ --- *
 *
 * Arguments:	@keyset *ks@ = pointer to a keyset
 *		@buf *b@ = pointer to input buffer
 *		@buf *bb@ = pointer to output buffer
 *
 * Returns:	Zero if OK, nonzero if the key needs replacing.  If the
 *		encryption failed, the output buffer is broken and zero is
 *		returned.
 *
 * Use:		Encrypts a block of data using the key.  Note that the `key
 *		ought to be replaced' notification is only ever given once
 *		for each key.  Also note that this call forces a keyset to be
 *		used even if it's marked as not for data output.
 */

extern int ks_encrypt(keyset */*ks*/, buf */*b*/, buf */*bb*/);

/* --- @ks_decrypt@ --- *
 *
 * Arguments:	@keyset *ks@ = pointer to a keyset
 *		@buf *b@ = pointer to an input buffer
 *		@buf *bb@ = pointer to an output buffer
 *
 * Returns:	Zero on success, or nonzero if there was some problem.
 *
 * Use:		Attempts to decrypt a message using a given key.  Note that
 *		requesting decryption with a key directly won't clear a
 *		marking that it's not for encryption.
 */

extern int ks_decrypt(keyset */*ks*/, buf */*b*/, buf */*bb*/);

/* --- @ksl_free@ --- *
 *
 * Arguments:	@keyset **ksroot@ = pointer to keyset list head
 *
 * Returns:	---
 *
 * Use:		Frees (releases references to) all of the keys in a keyset.
 */

extern void ksl_free(keyset **/*ksroot*/);

/* --- @ksl_link@ --- *
 *
 * Arguments:	@keyset **ksroot@ = pointer to keyset list head
 *		@keyset *ks@ = pointer to a keyset
 *
 * Returns:	---
 *
 * Use:		Links a keyset into a list.  A keyset can only be on one list
 *		at a time.  Bad things happen otherwise.
 */

extern void ksl_link(keyset **/*ksroot*/, keyset */*ks*/);

/* --- @ksl_prune@ --- *
 *
 * Arguments:	@keyset **ksroot@ = pointer to keyset list head
 *
 * Returns:	---
 *
 * Use:		Prunes the keyset list by removing keys which mustn't be used
 *		any more.
 */

extern void ksl_prune(keyset **/*ksroot*/);

/* --- @ksl_encrypt@ --- *
 *
 * Arguments:	@keyset **ksroot@ = pointer to keyset list head
 *		@buf *b@ = pointer to input buffer
 *		@buf *bb@ = pointer to output buffer
 *
 * Returns:	Nonzero if a new key is needed.
 *
 * Use:		Encrypts a packet.
 */

extern int ksl_encrypt(keyset **/*ksroot*/, buf */*b*/, buf */*bb*/);

/* --- @ksl_decrypt@ --- *
 *
 * Arguments:	@keyset **ksroot@ = pointer to keyset list head
 *		@buf *b@ = pointer to input buffer
 *		@buf *bb@ = pointer to output buffer
 *
 * Returns:	Nonzero if the packet couldn't be decrypted.
 *
 * Use:		Decrypts a packet.
 */

extern int ksl_decrypt(keyset **/*ksroot*/, buf */*b*/, buf */*bb*/);

/*----- Administration interface ------------------------------------------*/

/* --- @a_warn@ --- *
 *
 * Arguments:	@const char *fmt@ = pointer to format string
 *		@...@ = other arguments
 *
 * Returns:	---
 *
 * Use:		Informs all admin connections of a warning.
 */

extern void a_warn(const char */*fmt*/, ...);

/* --- @a_create@ --- *
 *
 * Arguments:	@int fd_in, fd_out@ = file descriptors to use
 *
 * Returns:	---
 *
 * Use:		Creates a new admin connection.
 */

extern void a_create(int /*fd_in*/, int /*fd_out*/);

/* --- @a_quit@ --- *
 *
 * Arguments:	---
 *
 * Returns:	---
 *
 * Use:		Shuts things down nicely.
 */

extern void a_quit(void);

/* --- @a_daemon@ --- *
 *
 * Arguments:	---
 *
 * Returns:	---
 *
 * Use:		Informs the admin module that it's a daemon.
 */

extern void a_daemon(void);

/* --- @a_init@ --- *
 *
 * Arguments:	@const char *sock@ = socket name to create
 *
 * Returns:	---
 *
 * Use:		Creates the admin listening socket.
 */

extern void a_init(const char */*sock*/);

/*----- Peer management ---------------------------------------------------*/

/* --- @p_txstart@ --- *
 *
 * Arguments:	@peer *p@ = pointer to peer block
 *		@unsigned msg@ = message type code
 *
 * Returns:	A pointer to a buffer to write to.
 *
 * Use:		Starts sending to a peer.  Only one send can happen at a
 *		time.
 */

extern buf *p_txstart(peer */*p*/, unsigned /*msg*/);

/* --- @p_txend@ --- *
 *
 * Arguments:	@peer *p@ = pointer to peer block
 *
 * Returns:	---
 *
 * Use:		Sends a packet to the peer.
 */

extern void p_txend(peer */*p*/);

/* --- @p_tun@ --- *
 *
 * Arguments:	@peer *p@ = pointer to peer block
 *		@buf *b@ = buffer containing incoming packet
 *
 * Returns:	---
 *
 * Use:		Handles a packet which needs to be sent to a peer.
 */

extern void p_tun(peer */*p*/, buf */*b*/);

/* --- @p_interval@ --- *
 *
 * Arguments:	---
 *
 * Returns:	---
 *
 * Use:		Called periodically to do tidying.
 */

extern void p_interval(void);

/* --- @p_stats@ --- *
 *
 * Arguments:	@peer *p@ = pointer to a peer block
 *
 * Returns:	A pointer to the peer's statistics.
 */

extern stats *p_stats(peer */*p*/);

/* --- @p_ifname@ --- *
 *
 * Arguments:	@peer *p@ = pointer to a peer block
 *
 * Returns:	A pointer to the peer's interface name.
 */

extern const char *p_ifname(peer */*p*/);

/* --- @p_addr@ --- *
 *
 * Arguments:	@peer *p@ = pointer to a peer block
 *
 * Returns:	A pointer to the peer's address.
 */

extern const addr *p_addr(peer */*p*/);

/* --- @p_init@ --- *
 *
 * Arguments:	@struct in_addr addr@ = address to bind to
 *		@unsigned port@ = port number to listen to
 *
 * Returns:	---
 *
 * Use:		Initializes the peer system; creates the socket.
 */

extern void p_init(struct in_addr /*addr*/, unsigned /*port*/);

/* --- @p_port@ --- *
 *
 * Arguments:	---
 *
 * Returns:	Port number used for socket.
 */

unsigned p_port(void);

/* --- @p_create@ --- *
 *
 * Arguments:	@const char *name@ = name for this peer
 *		@struct sockaddr *sa@ = socket address of peer
 *		@size_t sz@ = size of socket address
 *
 * Returns:	Pointer to the peer block, or null if it failed.
 *
 * Use:		Creates a new named peer block.  No peer is actually attached
 *		by this point.
 */

extern peer *p_create(const char */*name*/,
		      struct sockaddr */*sa*/, size_t /*sz*/);

/* --- @p_name@ --- *
 *
 * Arguments:	@peer *p@ = pointer to a peer block
 *
 * Returns:	A pointer to the peer's name.
 */

extern const char *p_name(peer */*p*/);

/* --- @p_find@ --- *
 *
 * Arguments:	@const char *name@ = name to look up
 *
 * Returns:	Pointer to the peer block, or null if not found.
 *
 * Use:		Finds a peer by name.
 */

extern peer *p_find(const char */*name*/);

/* --- @p_destroy@ --- *
 *
 * Arguments:	@peer *p@ = pointer to a peer
 *
 * Returns:	---
 *
 * Use:		Destroys a peer.
 */

extern void p_destroy(peer */*p*/);

/* --- @p_first@, @p_next@ --- *
 *
 * Arguments:	@peer *p@ = a peer block
 *
 * Returns:	@peer_first@ returns the first peer in some ordering;
 *		@peer_next@ returns the peer following a given one in the
 *		same ordering.  Null is returned for the end of the list.
 */

extern peer *p_first(void);
extern peer *p_next(peer */*p*/);

/*----- Tunnel interface --------------------------------------------------*/

/* --- @tun_init@ --- *
 *
 * Arguments:	---
 *
 * Returns:	---
 *
 * Use:		Initializes the tunneling system.  Maybe this will require
 *		opening file descriptors or something.
 */

extern void tun_init(void);

/* --- @tun_create@ --- *
 *
 * Arguments:	@tunnel *t@ = pointer to tunnel block
 *		@peer *p@ = pointer to peer block
 *
 * Returns:	Zero if it worked, nonzero on failure.
 *
 * Use:		Initializes a new tunnel.
 */

extern int tun_create(tunnel */*t*/, peer */*p*/);

/* --- @tun_ifname@ --- *
 *
 * Arguments:	@tunnel *t@ = pointer to tunnel block
 *
 * Returns:	A pointer to the tunnel's interface name.
 */

extern const char *tun_ifname(tunnel */*t*/);

/* --- @tun_inject@ --- *
 *
 * Arguments:	@tunnel *t@ = pointer to tunnel block
 *		@buf *b@ = buffer to send
 *
 * Returns:	---
 *
 * Use:		Injects a packet into the local network stack.
 */

extern void tun_inject(tunnel */*t*/, buf */*b*/);

/* --- @tun_destroy@ --- *
 *
 * Arguments:	@tunnel *t@ = pointer to tunnel block
 *
 * Returns:	---
 *
 * Use:		Destroys a tunnel.
 */

extern void tun_destroy(tunnel */*t*/);

/*----- Other handy utilities ---------------------------------------------*/

/* --- @mpstr@ --- *
 *
 * Arguments:	@mp *m@ = a multiprecision integer
 *
 * Returns:	A pointer to the integer's textual representation.
 *
 * Use:		Converts a multiprecision integer to a string.  Corrupts
 *		@buf_t@.
 */

extern const char *mpstr(mp */*m*/);

/* --- @timestr@ --- *
 *
 * Arguments:	@time_t t@ = a time to convert
 *
 * Returns:	A pointer to a textual representation of the time.
 *
 * Use:		Converts a time to a textual representation.  Corrupts
 *		@buf_t@.
 */

extern const char *timestr(time_t /*t*/);

/*----- That's all, folks -------------------------------------------------*/

#ifdef __cplusplus
  }
#endif

#endif
